unit DIBEditor;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: DIBEditor.PAS, released August 28, 2000.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2000 Peter Morris.
All Rights Reserved.

Purpose of file:
Editor for DIBs, load / save / import mask etc.

Contributor(s):
None as yet


Last Modified: August 31, 2000
Current Version: 1.00

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org
or at http://www.stuckindoors.com/dib


Known Issues:
Add an Eyedropper for chosing transparent colour.


Date : 31 - Aug, 2000 :
By :pete@stuckindoors.com
Changes
Made TxxxxxxEditor to TxxxxxxxProperty to comply with VCL standards.
Made sure all unit names start with DIB to avoice conflicts with other people's
component packs.

-----------------------------------------------------------------------------}


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  cDIBControl, cDIBImage, StdCtrls, cDIBPanel, ExtCtrls, Menus, ExtDlgs,
  Spin, jpeg, ComCtrls, DSGNINTF, TypInfo, cDIB, cDIBImageList;

type
  TAbstractSuperDIBProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes : TPropertyAttributes; override;
  end;

  TfmDIBEditor = class(TForm)
    MainMenu1: TMainMenu;
    Image1: TMenuItem;
    Mask1: TMenuItem;
    Import1: TMenuItem;
    Import2: TMenuItem;
    pnlControl: TPanel;
    gbTransparency: TGroupBox;
    cbTransparent: TCheckBox;
    Label3: TLabel;
    Label4: TLabel;
    btnOK: TButton;
    btnCancel: TButton;
    opd1: TOpenPictureDialog;
    cbTransparentMode: TComboBox;
    shTransparentColor: TShape;
    cdColor: TColorDialog;
    DIBImageList1: TDIBImageList;
    dicRender: TDIBImageContainer;
    DIBImage1: TDIBImage;
    cbMasked: TCheckBox;
    Panel1: TPanel;
    Label1: TLabel;
    seAngle: TSpinEdit;
    pbAngle: TProgressBar;
    Label2: TLabel;
    seOpacity: TSpinEdit;
    pbOpacity: TProgressBar;
    Label5: TLabel;
    seScale: TSpinEdit;
    pbScale: TProgressBar;
    Label6: TLabel;
    procedure Import1Click(Sender: TObject);
    procedure seAngleChange(Sender: TObject);
    procedure seOpacityChange(Sender: TObject);
    procedure cbMaskedClick(Sender: TObject);
    procedure Import2Click(Sender: TObject);
    procedure seScaleChange(Sender: TObject);
    procedure cbTransparentClick(Sender: TObject);
    procedure pbAngleMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure pbAngleMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbAngleMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure pbOpacityMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure pbScaleMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure cbTransparentModeChange(Sender: TObject);
    procedure shTransparentColorMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
    FCaptureMouse   : Boolean;
    procedure ClipCurs(AComponent : TControl);
  public
    { Public declarations }
    TheImage        : TMemoryDIB;
  end;

implementation

uses DIBImages;

{$R *.DFM}

{ TAbstractSuperDIBProperty }

procedure TAbstractSuperDIBProperty.Edit;
var
  EdForm            : TfmDIBEditor;
begin
  EdForm := TfmDIBEditor.Create(Application);
  with EdForm do
  try
    TheImage := DIBImageList1.DIBImages[0].DIB;
    with TheImage do begin
      Assign(TAbstractSuperDIB(GetOrdValue));
      seAngle.Value := Angle;
      pbAngle.Position := Angle;
      seOpacity.Value := Opacity;
      pbOpacity.Position := Opacity;
      seScale.Value := Round(Scale);
      pbScale.Position := seScale.Value;
      cbMasked.Checked := Masked;
      cbTransparent.Checked := Transparent;
      cbTransparentMode.ItemIndex := Integer(TransparentMode);
      shTransparentColor.Brush.Color := TransparentColor;
    end;

    ShowModal;

    if ModalResult = mrOK then with TAbstractSuperDIB(GetOrdValue) do begin
      Assign(TheImage);
      Self.Designer.Modified;
    end;
  finally
    EdForm.Free;
  end;
end;

function TAbstractSuperDIBProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paReadOnly, paDialog];
end;


{ TfmDIBEditor }

procedure TfmDIBEditor.Import1Click(Sender: TObject);
begin
  if opd1.execute then
    TheImage.ImportPicture(opd1.Filename);
end;

procedure TfmDIBEditor.seAngleChange(Sender: TObject);
begin
  TheImage.Angle := seAngle.Value;
  DIBImage1.Angle := seAngle.Value;
  pbAngle.Position := seAngle.Value;
end;

procedure TfmDIBEditor.seOpacityChange(Sender: TObject);
begin
  TheImage.Opacity := seOpacity.Value;
  DIBImage1.Opacity := seOpacity.Value;
  pbOpacity.Position := seOpacity.Value;
end;

procedure TfmDIBEditor.cbMaskedClick(Sender: TObject);
begin
  TheImage.Masked := cbMasked.Checked;
end;

procedure TfmDIBEditor.Import2Click(Sender: TObject);
begin
  if opd1.Execute then
    TheImage.ImportMask(opd1.Filename);
end;

procedure TfmDIBEditor.seScaleChange(Sender: TObject);
begin
  TheImage.Scale := seScale.Value;
  DIBImage1.Scale := seScale.Value;
  pbScale.Position := seScale.Value;
end;

procedure TfmDIBEditor.cbTransparentClick(Sender: TObject);
begin
  TheImage.Transparent := cbTransparent.Checked;
end;

procedure TfmDIBEditor.pbAngleMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FCaptureMouse then seAngle.Value := Trunc(X / pbAngle.Width * 359);
end;

procedure TfmDIBEditor.FormCreate(Sender: TObject);
begin
  FCaptureMouse := False;
  TheImage := DIBImageList1.DIBImages[0].DIB;
  DIBImage1.DIBImageList := DIBImageList1;
  DIBImage1.IndexMain.DIBIndex :=0;
end;

procedure TfmDIBEditor.pbAngleMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FCaptureMouse := True;
  ClipCurs(TControl(Sender));
end;

procedure TfmDIBEditor.ClipCurs(AComponent: TControl);
var
  R                 : TRect;
begin
  R := AComponent.ClientRect;
  R.TopLeft := AComponent.ClientToScreen(R.TopLeft);
  R.Bottom := R.Bottom + R.Top;
  R.Right := R.Right + R.Left;
  ClipCursor(@R);
end;

procedure TfmDIBEditor.pbAngleMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  FCaptureMouse := False;
  ClipCursor(nil);
end;

procedure TfmDIBEditor.pbOpacityMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
  if FCaptureMouse then seOpacity.Value := Trunc(X / pbOpacity.Width * 255);
end;

procedure TfmDIBEditor.pbScaleMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  if FCaptureMouse then seScale.Value := Trunc(X / pbScale.Width * pbScale.Max)+1;
end;

procedure TfmDIBEditor.cbTransparentModeChange(Sender: TObject);
begin
  TheImage.TransparentMode := TTransparentMode(cbTransparentMode.ItemIndex);
  shTransparentColor.Brush.Color := TheImage.TransparentColor;
end;

procedure TfmDIBEditor.shTransparentColorMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if cdColor.Execute then begin
    shTransparentColor.Brush.Color := cdColor.Color;
    TheImage.TransparentColor := cdColor.Color;
  end;
end;

end.
