unit cDIBStandardCompressors;

{-----------------------------------------------------------------------------
The contents of this file are subject to the Mozilla Public License
Version 1.1 (the "License"); you may not use this file except in compliance
with the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/MPL-1.1.html

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either expressed or implied. See the License for
the specific language governing rights and limitations under the License.

The Original Code is: cDIBStandardCompressors.PAS, released September 04, 2000.

The Initial Developer of the Original Code is Peter Morris (pete@stuckindoors.com),
Portions created by Peter Morris are Copyright (C) 2000 Peter Morris.
All Rights Reserved.

Purpose of file:
To provide at least 1 working example of a DIBCompressor

Contributor(s):
None as yet


Last Modified: September 04, 2000
Current Version: 1.00

You may retrieve the latest version of this file at the Project JEDI home page,
located at http://www.delphi-jedi.org
or at http://www.stuckindoors.com/dib


Known Issues:
-----------------------------------------------------------------------------}


interface
uses
  cDIBCompressor, Classes, Windows, Dialogs, SysUtils;

type
  TRLEDIBCompressor = class(TAbstractDIBCompressor)
  private
    function  RLE(const Source, Dest; const SourceSize : DWord) : DWord;
    procedure Shuffle(const Source, Dest; const SourceSize : DWord);
    procedure UnRLE(const Source, Dest; const SourceSize : DWord);
    procedure UnShuffle(const Source, Dest; const SourceSize : Dword);
  public
    function  GetAboutText : String; override;
    function  GetAuthor : String; override;
    function  GetEmail : String; override;
    function  GetURL : String; override;

    function  GetGUID : TGUID; override;
    function  CanDecompress(const GUID : TGUID) : Boolean; override;
    function  Compress(const Source, Dest; const SourceSize : DWord) : DWord; override;
    procedure Decompress(const GUID : TGUID; const Source, Dest; const SourceSize,DestSize : DWord); override;
    function  GetDisplayName : String; override;
  end;

implementation
uses
  ComObj;
  
const
  cRLEGUID : TGUID = '{CABA9056-7349-43A4-B010-3764FBFAC230}';

{ TRLEDIBCompressor }

function TRLEDIBCompressor.CanDecompress(const GUID: TGUID): Boolean;
begin
  Result := (GUIDToString(GUID) = GUIDToString(cRLEGUID));
end;

function TRLEDIBCompressor.Compress(const Source, Dest;
  const SourceSize : DWord) : DWord;
var
  pDest                       : Pointer;
begin
  Getmem(pDest, SourceSize);
  try
    Shuffle(Source, pDest^, SourceSize);
    Result := RLE(pDest^, Dest, SourceSize);
  finally
    Freemem(pDest);
  end;
end;

(*
var
  RepeatValue,
  Value,
  ByteCount                   : Byte;
  MS                          : TMemoryStream;
begin
  Result := True;
  MS := TMemoryStream.Create;
  try
    Shuffle(Source, MS);
    MS.Seek(0, soFromBeginning);
    MS.Read(RepeatValue,1);

    ByteCount := 0;
    while (MS.Position < MS.Size) do begin
      Value := RepeatValue;
      while (RepeatValue = Value) and
            (ByteCount < 255) and
            (MS.Position < MS.Size) do
      begin
        Inc(ByteCount);
        MS.Read(RepeatValue,1);
      end;

      Dest.Write(ByteCount, 1);
      Dest.Write(Value, 1);
      ByteCount := 0;

      //If half way through and size is larger, then exit
      if (Source.Position > Source.Size div 2) and (Dest.Size > Source.Position) then begin
        Result := False;
        Break
      end;
    end;
    //In the end, if the result is larger, then fail
    if Dest.Size > Source.Size then Result := False;
  finally
    MS.Free;
  end;
end;
*)

procedure TRLEDIBCompressor.Decompress(const GUID : TGUID; const Source, Dest;
  const SourceSize,DestSize : DWord);
var
  pDest                       : Pointer;
begin
  Getmem(pDest, DestSize);
  try
    UnRLE(Source, pDest^, SourceSize);
    UnShuffle(pDest^, Dest, DestSize);
  finally
    Freemem(pDest);
  end;
end;

function TRLEDIBCompressor.GetAboutText: String;
begin
  Result := 'Shuffle4 RLE' + #13#10 +
            #13#10+
            'Shuffles the RGBA values from' +#13#10+
            'RGBA,RGBA,RGBA,RGBA to' +#13#10+
            'RRRR,GGGG,BBBB,AAAA' + #13#10+
            'in order to create higher repitition,' +#13#10+
            'and then performs a simple RLE compression';
end;

function TRLEDIBCompressor.GetAuthor: String;
begin
  Result := 'Peter Morris';
end;

function TRLEDIBCompressor.GetDisplayName: String;
begin
  Result := 'Shuffle4 RLE';
end;

function TRLEDIBCompressor.GetEmail: String;
begin
  Result := 'Pete@StuckIndoors.com';
end;

function TRLEDIBCompressor.GetGUID: TGUID;
begin
  Result := cRLEGUID;
end;

function TRLEDIBCompressor.GetURL: String;
begin
  Result := 'http://www.stuckindoors.com/dib';
end;

function TRLEDIBCompressor.RLE(const Source; const Dest;
  const SourceSize: DWord) : DWord; assembler;
asm
      push ESI
      push EDI
      push EBX
      push Dest

      mov  ESI, Source
      mov  EDI, Dest
      //Read bytes left
      mov  ECX, SourceSize
      //Write bytes left
      mov  EBX, SourceSize

      xor  dl, dl //Byte count
      mov  ah, [ESI] // Repeat value

      //each write to Dest takes 2 bytes
      shr  EBX, 1

  @OuterLoop:
      mov  al, ah //Value = repeat value
  @InnerLoop:
      inc  dl
      inc  ESI
      dec  ECX
      jz   @WriteValues
      cmp  dl, 255
      jz   @WriteValues

      mov  ah, [ESI] //Get the next value
      cmp  ah, al //compare
      jnz  @WriteValues //if <> then write values
      jmp  @InnerLoop

  @WriteValues:
      mov  [EDI], dl
      inc  EDI
      xor  dl, dl
      mov  [EDI], al
      inc  EDI
      cmp  ECX, 0
      jz   @Compressed
      //If we are here, we are not at the end of SOURCE
      //Dec EBX (Dest bytes remaining), if 0 then quit
      dec  EBX
      jz   @NotCompressed
      jmp  @OuterLoop

  //We reached the end of the destination buffer before the
  //end of the source buffer, in other words, Dest is bigger than Source
  @NotCompressed:
      //pop the extra push that we no longer need
      pop  EDI

      mov  Result, 0
      jmp  @TheEnd

  @Compressed:
      //Get the original @Dest into ESI
      pop  ESI
      //See how far it is from ESI -> EDI
      sub  EDI, ESI
      mov  Result, EDI
  @TheEnd:
  //pop Source
      pop  EBX
      pop  EDI
      pop  ESI

end;

procedure TRLEDIBCompressor.Shuffle(const Source; const Dest;
  const SourceSize: DWord); assembler;
asm;
      push ESI
      push EDI
      push EBX

      mov  EBX, Source
      mov  EDX, SourceSize
      mov  EDI, Dest

      //Alpha
      mov  ECX, EDX
      mov  ESI, EBX
      inc  EBX
      shr  ECX, 2
  @AlphaLoop:
      mov  al, [ESI]
      lea  ESI, [ESI+4]
      mov  [EDI], al
      inc  EDI
      dec  ECX
      jnz  @AlphaLoop

      //Blue
      mov  ECX, EDX
      mov  ESI, EBX
      inc  EBX
      shr  ECX, 2
  @BlueLoop:
      mov  al, [ESI]
      lea  ESI, [ESI+4]
      mov  [EDI], al
      inc  EDI
      dec  ECX
      jnz  @BlueLoop

      //Green
      mov  ECX, EDX
      mov  ESI, EBX
      inc  EBX
      shr  ECX, 2
  @GreenLoop:
      mov  al, [ESI]
      lea  ESI, [ESI+4]
      mov  [EDI], al
      inc  EDI
      dec  ECX
      jnz  @GreenLoop

      //Red
      mov  ECX, EDX
      mov  ESI, EBX
      inc  EBX
      shr  ECX, 2
  @RedLoop:
      mov  al, [ESI]
      lea  ESI, [ESI+4]
      mov  [EDI], al
      inc  EDI
      dec  ECX
      jnz  @RedLoop

      pop  EBX
      pop  EDI
      pop  ESI

end;

procedure TRLEDIBCompressor.UnRLE(const Source, Dest;
  const SourceSize: DWord); assembler;
asm;
      push ESI
      push EDI
      push EBX

      mov  EBX, SourceSize
      mov  ESI, Source
      mov  EDI, Dest
      xor  ECX, ECX
      shr  EBX, 1
  @ReadLoop:
      mov  CL, [ESI]
      inc  ESI
      LodSB
      repnz StoSB

      dec  EBX
      jnz  @ReadLoop

      pop  EBX
      pop  EDI
      pop  ESI
end;

//The idea is we read 1 byte
//write 1 byte
//skip the dest forward 3 pixels
procedure TRLEDIBCompressor.UnShuffle(const Source, Dest;
  const SourceSize: Dword); assembler;
asm;
      push ESI
      push EDI
      push EBX

      mov  ESI, Source
      mov  EBX, Dest
      mov  EDX, SourceSize


      //Alpha
      mov  ECX, EDX
      mov  EDI, EBX
      inc  EBX
      shr  ECX, 2
  @AlphaLoop:
      mov  al, [ESI]
      inc  ESI
      mov  [EDI], al
      lea  EDI, [EDI+4]
      dec  ECX
      jnz  @AlphaLoop

      //Blue
      mov  ECX, EDX
      mov  EDI, EBX
      inc  EBX
      shr  ECX, 2
  @BlueLoop:
      mov  al, [ESI]
      inc  ESI
      mov  [EDI], al
      lea  EDI, [EDI+4]
      dec  ECX
      jnz  @BlueLoop

      //Green
      mov  ECX, EDX
      mov  EDI, EBX
      inc  EBX
      shr  ECX, 2
  @GreenLoop:
      mov  al, [ESI]
      inc  ESI
      mov  [EDI], al
      lea  EDI, [EDI+4]
      dec  ECX
      jnz  @GreenLoop

      //Red
      mov  ECX, EDX
      mov  EDI, EBX
      inc  EBX
      shr  ECX, 2
  @RedLoop:
      mov  al, [ESI]
      inc  ESI
      mov  [EDI], al
      lea  EDI, [EDI+4]
      dec  ECX
      jnz  @RedLoop


      pop  EBX
      pop  EDI
      pop  ESI


end;

initialization
  RegisterDIBCompressor(TRLEDIBCompressor.Create);

end.
